/**

    Copyright 2004 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

**/
package it.eng.spago.base;

import it.eng.spago.dispatching.httpchannel.AdapterPortlet;
import it.eng.spago.tracing.TracerSingleton;

import javax.portlet.PortletRequest;
import javax.servlet.http.HttpServletRequest;
/**
 * La classe <code>RequestContainerPortletAccess</code> permette di accede all'oggetto
 * <code>RequestContainer</code> all'interno di una portlet<p>
 * @version 1.0, 20/01/2005
 * @author Daniela Butano
 * @see BaseContainer
 * @see ResponseContainer
 */
public class RequestContainerPortletAccess {
    /**
     * Ritorna il <code>RequestContainer</code> il cui riferimento &egrave;
     * memorizzato nell'oggetto <code>RenderRequest</code> <em>request</em>
     * della richiesta corrente.
     * <p>
     * @param request <code>RenderRequest</code> della richiesta corrente
     * @return <code>RequestContainer</code> il contenitore dei dati
     */
	 private static RequestContainer getRequestContainer(PortletRequest renderRequest) {
        if (renderRequest == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "RequestContainerPortletAccess::getRequestContainer: renderRequest nullo");
            return null;
        } // if (renderRequest == null)
        String requestContainerName = (String)renderRequest.getPortletSession().getAttribute(AdapterPortlet.REQUEST_CONTAINER_NAME);
        //String requestContainerName = renderRequest.getParameter(Constants.REQUEST_CONTAINER);
        if (requestContainerName == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "RequestContainerPortletAccess::getRequestContainer: requestContainerName nullo");
            return null;
        } // if (requestContainerName == null)
        RequestContainer requestContainer = (RequestContainer)renderRequest.getPortletSession().getAttribute(requestContainerName);
        if (requestContainer == null)
               TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                   "RequestContainerPortletAccess::getRequestContainer: requestContainer nullo");
        return requestContainer;
    } // public static RequestContainer getRequestContainer(HttpServletRequest request) 
	
	 
	 
	public static RequestContainer getRequestContainer(HttpServletRequest request) {
        if (request == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "RequestContainerPortletAccess::getRequestContainer: request nullo");
            return null;
        } // if (request == null)
        PortletRequest renderRequest = (PortletRequest)request.getAttribute("javax.portlet.request");
        return getRequestContainer(renderRequest); 
    } // public static RequestContainer getRequestContainer(HttpServletRequest request) 

	
	
	
	private static void delRequestContainer(PortletRequest renderRequest) {
        if (renderRequest == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "RequestContainerPortletAccess::delRequestContainer: renderRequest nullo");
            return;
        } // if (renderRequest == null)
        String requestContainerName = (String)renderRequest.getPortletSession().getAttribute(AdapterPortlet.REQUEST_CONTAINER_NAME);
        //String requestContainerName = renderRequest.getParameter(Constants.REQUEST_CONTAINER);
        if (requestContainerName == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "RequestContainerPortletAccess::delRequestContainer: requestContainerName nullo");
            return;
        } // if (requestContainerName == null)  
        renderRequest.getPortletSession().setAttribute(requestContainerName, null);
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "RequestContainerPortletAccess::delRequestContainer: requestContainer rimosso");
    } // public static void delRequestContainer(HttpServletRequest request)
	
	
	
	
	
    public static void delRequestContainer(HttpServletRequest request) {
        if (request == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "RequestContainerPortletAccess::delRequestContainer: request nullo");
            return;
        } // if (request == null)
        PortletRequest renderRequest = (PortletRequest)request.getAttribute("javax.portlet.request");
        delRequestContainer(renderRequest);       
    } // public static void delRequestContainer(HttpServletRequest request)
	
    
   
} // public class RequestContainerPortletAccess
