/**

    Copyright 2004 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

**/
package it.eng.spago.base;

import it.eng.spago.dispatching.httpchannel.AdapterPortlet;
import it.eng.spago.tracing.TracerSingleton;

import javax.portlet.PortletContext;
import javax.portlet.PortletRequest;
import javax.servlet.http.HttpServletRequest;
/**
 * La classe <code>ResponseContainerPortletAccess</code> permette di accede all'oggetto
 * <code>ResponseContainer</code> all'interno di una portlet<p>
 * @version 1.0, 20/01/2005
 * @author Daniela Butano
 * @see BaseContainer
 * @see ResponseContainer
 */
public class ResponseContainerPortletAccess {
    /**
     * Ritorna il <code>ResponseContainer</code> il cui riferimento &egrave;
     * memorizzato nell'oggetto <code>RenderRequest</code> <em>request</em>
     * della richiesta corrente.
     * <p>
     * @param request <code>HttpServletRequest</code> della richiesta corrente
     * @return <code>ResponseContainer</code> il contenitore dei dati
     */
    public static ResponseContainer getResponseContainer(HttpServletRequest request) {
        if (request == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "ResponseContainerPortletAccess::getResponseContainer: request nullo");
            return null;
        } // if (request == null)
        PortletRequest renderRequest = (PortletRequest)request.getAttribute("javax.portlet.request");
        if (renderRequest == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "ResponseContainerPortletAccess::getResponseContainer: renderRequest nullo");
            return null;
        } // if (renderRequest == null)
        
        String responseContainerName = (String)renderRequest.getPortletSession().getAttribute(AdapterPortlet.RESPONSE_CONTAINER_NAME);
		//String responseContainerName = (String)request.getAttribute(Constants.RESPONSE_CONTAINER); 
    	if( responseContainerName==null ) {
			responseContainerName = request.getParameter(Constants.RESPONSE_CONTAINER);
    	} else {
    		request.removeAttribute(Constants.REQUEST_CONTAINER);
    		request.removeAttribute(Constants.RESPONSE_CONTAINER);
    	}
        
        // String responseContainerName = renderRequest.getParameter(Constants.RESPONSE_CONTAINER);
        if (responseContainerName == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "ResponseContainerPortletAccess::getResponseContainer: responseContainerName nullo");
            return null;
        } // if (requestContainerName == null)
        ResponseContainer responseContainer = (ResponseContainer)renderRequest.getPortletSession().getAttribute(responseContainerName);
        if (responseContainer == null)
               TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                   "ResponseContainerPortletAccess::getResponseContainer: responseContainer nullo");
        return responseContainer;
    } // public static ResponseContainer getResponseContainer(HttpServletRequest request) 
    
    
    public static void delResponseContainer(HttpServletRequest request) {
        if (request == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "ResponseContainerPortletAccess::delResponseContainer: request nullo");
            return;
        } // if (request == null)
        PortletRequest renderRequest = (PortletRequest)request.getAttribute("javax.portlet.request");
        if (renderRequest == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                "ResponseContainerPortletAccess::delResponseContainer: renderRequest nullo");
            return;
        } // if (renderRequest == null)
        String responseContainerName = (String)renderRequest.getPortletSession().getAttribute(AdapterPortlet.RESPONSE_CONTAINER_NAME);
        //String responseContainerName = renderRequest.getParameter(Constants.RESPONSE_CONTAINER);
        if (responseContainerName == null) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "ResponseContainerPortletAccess::delResponseContainer: responseContainerName nullo");
            return;
        } // if (responseContainerName == null)  
        renderRequest.getPortletSession().setAttribute(responseContainerName, null);
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "ResponseContainerPortletAccess::delResponseContainer: responseContainer rimosso");
    } // public static void delResponseContainer(HttpServletRequest request)
} // public class ResponseContainerPortletAccess
