/**

    Copyright 2004 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

**/
package it.eng.spago.dispatching.httpchannel;

import it.eng.spago.base.Constants;
import it.eng.spago.base.PortletAccess;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.action.NavigationErrorUtility;
import it.eng.spago.dispatching.action.SessionExpiredUtility;
import it.eng.spago.dispatching.coordinator.CoordinatorIFace;
import it.eng.spago.dispatching.coordinator.DispatcherManager;
import it.eng.spago.dispatching.service.DefaultRequestContext;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.init.PortletInitializerManager;
import it.eng.spago.navigation.NavigationException;
import it.eng.spago.navigation.Navigator;
import it.eng.spago.navigation.LightNavigationManager;
import it.eng.spago.presentation.PresentationRendering;
import it.eng.spago.presentation.Publisher;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.PortletTracer;
import it.eng.spago.util.Serializer;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.Portlet;
import javax.portlet.PortletException;
import javax.portlet.PortletMode;
import javax.portlet.PortletRequest;
import javax.portlet.PortletResponse;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

public class AdapterPortlet extends GenericPortlet {
    public static final String NEW_SESSION = "NEW_SESSION";
    public static final String HTTP_CONTENT_TYPE = "text/html";
    public static final String WAP_CONTENT_TYPE = "text/vnd.wap.wml";
    public static final String HTTP_ACCEPT_HEADER = "ACCEPT";
    public static final String WAP_MIME_TYPE = "vnd.wap";
    public static final String HTTP_SESSION_ID = "HTTP_SESSION_ID";
    private static final String SERVLET_PUBLISHER_TYPE = "SERVLET";
    private static final String JSP_PUBLISHER_TYPE = "JSP";
    private static final String FORWARD_PUBLISHING_MODE = "FORWARD";
	private static final String SERIALIZE_SESSION_ATTRIBUTE = "COMMON.SERIALIZE_SESSION";
	private static final String SERVICE_EXCEPTION = "SERVICE_EXCEPTION";
	private static final String PORTLET_MODE = "PORTLET_MODE";
	private static final String PORTLET_EXCEPTION = "PORTLET_EXCEPTION";
	private static final String INITIAL_JSP = "pageStart";
	public static final String REQUEST_CONTAINER_NAME = "REQUEST_CONTAINER_NAME";
	public static final String RESPONSE_CONTAINER_NAME = "RESPONSE_CONTAINER_NAME";
	private static final String LOOPBACK = "LOOPBACK";
	private boolean serializeSession = false;
	private static final String SERVICE_TYPE = "serviceType";
	private static final String SERVICE_NAME = "serviceName";
	private static final String SERVICE_ACTION = "ACTION";
	private static final String SERVICE_PAGE = "PAGE";
	private static final String SERVICE_JSP = "JSP";
	
	
	
	
	
	public void init() throws PortletException {
		super.init();
		
		PortletTracer.info(Constants.NOME_MODULO, "AdapterPortlet", "init", "Invocato");
		String serializeSessionStr = (String) ConfigSingleton.getInstance().getAttribute(SERIALIZE_SESSION_ATTRIBUTE);
		if ((serializeSessionStr != null) && (serializeSessionStr.equalsIgnoreCase("TRUE"))) {
			serializeSession = true;
		}
		PortletInitializerManager.init();
	}

	
	
    public void processAction(
        ActionRequest request,
        ActionResponse response)
        throws PortletException, IOException {
       
    	PortletTracer.info(Constants.NOME_MODULO, "AdapterPortlet", "action", "Invocato");
        // set into threadLocal variables the jsr 168 portlet object 
        PortletAccess.setPortletConfig(getPortletConfig());
        PortletAccess.setPortletRequest(request);
        PortletAccess.setPortletResponse(response);
        PortletSession portletSession = request.getPortletSession();
        portletSession.setAttribute("BrowserLocale", request.getLocale());
        
        
        processService(request, response);
    }   
        
    
    
    
    
    public Object processService(PortletRequest request, PortletResponse response) throws PortletException, IOException {
    	HashMap map = new HashMap();
    	Monitor monitor = null;
    	PortletTracer.info(Constants.NOME_MODULO, "AdapterPortlet", "processService", "Invocato");
        try {
            SourceBean serviceRequest = null;
            EMFErrorHandler emfErrorHandler = null;
            // try to get from the request the loopback attribute. If present the method has to serve
            // a loopback request (the method has been called from the doRenderService)
            // if not present is a normal request
            boolean isLoop = request.getAttribute(LOOPBACK) != null;
            if(isLoop) {
            	// remove from the request the loopback attribute
            	request.removeAttribute(LOOPBACK);
            	//String responseContainerName = (String)request.getAttribute(Constants.RESPONSE_CONTAINER);
            	// get from the session the previous response container name
            	String responseContainerName = (String)request.getPortletSession().getAttribute(RESPONSE_CONTAINER_NAME);
            	// get from the session the previous response container
            	ResponseContainer loopbackResponseContainer = (ResponseContainer)request.getPortletSession().getAttribute(responseContainerName);
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.DEBUG,
                    "AdapterPortlet::service: loop-back rilevato");
                serviceRequest =
                    loopbackResponseContainer.getLoopbackServiceRequest();
                if (serviceRequest == null)
                    serviceRequest = new SourceBean(Constants.SERVICE_REQUEST);
                else {
                    Object newServiceRequest =
                        serviceRequest.getAttribute(Constants.SERVICE_REQUEST);
                    if ((newServiceRequest != null)
                        && (newServiceRequest instanceof SourceBean))
                        serviceRequest = (SourceBean) newServiceRequest;
                } // if (serviceRequest == null)
                emfErrorHandler = loopbackResponseContainer.getErrorHandler();
                if (emfErrorHandler == null)
                    emfErrorHandler = new EMFErrorHandler();
            } // if (isLoop)
            else {
	            monitor = MonitorFactory.start("controller.adapter.portlet");
	            serviceRequest = new SourceBean(Constants.SERVICE_REQUEST);
	            Enumeration names = request.getParameterNames();
	            while (names.hasMoreElements()) {
	                String parameterName = (String) names.nextElement();
	                String[] parameterValues =
	                    request.getParameterValues(parameterName);
	                if (parameterValues != null)
	                    for (int i = 0; i < parameterValues.length; i++)
	                        serviceRequest.setAttribute(
	                            parameterName,
	                            parameterValues[i]);
	            } // while (names.hasMoreElements())
	          	// ***************** START SERVICE ***********************************************
	            String actionName = (String)request.getAttribute("ACTION_NAME");
	            if(actionName!=null) {
	            	request.removeAttribute("ACTION_NAME");
	            	serviceRequest.setAttribute("ACTION_NAME", actionName);
	            	serviceRequest.setAttribute(NEW_SESSION, "TRUE");
	            }
	            String page = (String)request.getAttribute("PAGE");
	            if(page!=null) {
	            	request.removeAttribute("PAGE");
	            	serviceRequest.setAttribute("PAGE", page);
	            	serviceRequest.setAttribute(NEW_SESSION, "TRUE");
	            }  
	        	// *******************************************************************************************
	            emfErrorHandler = new EMFErrorHandler();
            } 
            //***************** NAVIGATION CONTROL *******************************************************
            String navigation = getInitParameter("light_navigation");
            if ("enabled".equalsIgnoreCase(navigation)) {
            	serviceRequest = LightNavigationManager.controlLightNavigation(request, serviceRequest);
            }
            //********************************************************************************************
            RequestContainer requestContainer = new RequestContainer();
            RequestContainer.setRequestContainer(requestContainer);
            requestContainer.setServiceRequest(serviceRequest);
            boolean isRequestedSessionIdValid = true;
            PortletSession session = request.getPortletSession(true);
            if (session.isNew()) {
                String newSessionString =
                    (String) (serviceRequest.getAttribute(NEW_SESSION));
                isRequestedSessionIdValid =
                    ((newSessionString != null)
                        && (newSessionString.equalsIgnoreCase("TRUE")));
            } // if (session.isNew())
            synchronized (session) {
                
            	// try to get the previous request container. Download from the session the previous 
            	// request container name and if it isn't null (it's null only formthe first invocation) 
            	//use it for download the request container object 
            	RequestContainer parentRequestContainer = null;
            	String parentRequestContainerName =
            		(String) session.getAttribute(REQUEST_CONTAINER_NAME);
            	if(parentRequestContainerName != null) {
            		parentRequestContainer = (RequestContainer) session.getAttribute(parentRequestContainerName);
            	}

                if (!Navigator.isNavigatorEnabled())
                    if (parentRequestContainer == null)
                        requestContainer.setSessionContainer(
                            new SessionContainer(true));
                    else
                        requestContainer.setSessionContainer(
                            parentRequestContainer.getSessionContainer());
                else {
                    if (parentRequestContainer == null)
                        requestContainer.setSessionContainer(
                            new SessionContainer(true));
                    else {
                        requestContainer.setSessionContainer(
                            new SessionContainer(false));
                        requestContainer.setParent(parentRequestContainer);
                    } // if (parentRequestContainer == null) else
                } // if (!Navigator.isNavigatorEnabled())
                session.setAttribute(Constants.REQUEST_CONTAINER, requestContainer);
            } // synchronized (session)
            if (!isRequestedSessionIdValid) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.WARNING,
                    "AdapterPortlet::processAction: sessione scaduta !");
                SessionExpiredUtility.setSessionExpiredAction(serviceRequest);
            } // if (!isRequestedSessionIdValid)
            
            requestContainer.setAttribute(
                    HTTP_SESSION_ID,
                    request.getPortletSession().getId());
            String channelType = "PORTLET";
            String channelTypeParameter =
                (String) (serviceRequest.getAttribute(Constants.CHANNEL_TYPE));
            String channelTypeHeader =
                (String) (requestContainer.getAttribute(HTTP_ACCEPT_HEADER));
            if (((channelTypeParameter != null)
                && channelTypeParameter.equalsIgnoreCase(Constants.WAP_CHANNEL))
                || ((channelTypeHeader != null)
                    && (channelTypeHeader.indexOf(WAP_MIME_TYPE) != -1)))
                channelType = Constants.WAP_CHANNEL;
            requestContainer.setChannelType(channelType);


            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "AdapterPortlet::processAction: requestContainer",
                requestContainer);
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "AdapterPortlet::processAction: sessionContainer",
                requestContainer.getSessionContainer());
            ResponseContainer responseContainer = new ResponseContainer();
            responseContainer.setErrorHandler(emfErrorHandler);
            SourceBean serviceResponse =
                new SourceBean(Constants.SERVICE_RESPONSE);
            responseContainer.setServiceResponse(serviceResponse);
            try {
                Navigator.checkNavigation(requestContainer);
            } // try
            catch (NavigationException ne) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.CRITICAL,
                    "AdapterPortlet::processAction: ",
                    ne);
                requestContainer.setServiceRequest(
                    NavigationErrorUtility.getNavigationErrorServiceRequest());
            } // catch (NavigationException ne)
            serviceRequest = requestContainer.getServiceRequest();
            RequestContextIFace requestContext =
                new DefaultRequestContext(requestContainer, responseContainer);
            CoordinatorIFace coordinator =
                DispatcherManager.getCoordinator(requestContext);
            Exception serviceException = null;
            if (coordinator == null) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.WARNING,
                    "AdapterPortlet::processAction: coordinator nullo !");
                serviceException = new Exception("Coordinatore non trovato");
                emfErrorHandler.addError(
                    new EMFInternalError(
                        EMFErrorSeverity.ERROR,
                        "Coordinatore non trovato !"));
            } // if (coordinator == null)
            else {
                ((RequestContextIFace) coordinator).setRequestContext(
                    requestContext);
                try {
                    coordinator.service(serviceRequest, serviceResponse);
                } // try
                catch (Exception ex) {
                    TracerSingleton.log(
                        Constants.NOME_MODULO,
                        TracerSingleton.CRITICAL,
                        "AdapterPortlet::processAction:",
                        ex);
                    serviceException = ex;
                    emfErrorHandler.addError(
                        new EMFInternalError(EMFErrorSeverity.ERROR, ex));
                    responseContainer.setAttribute(PORTLET_EXCEPTION, serviceException);
                } // catch (Exception ex)
                 ((RequestContextIFace) coordinator).setRequestContext(null);
                responseContainer.setBusinessType(
                    coordinator.getBusinessType());
                responseContainer.setBusinessName(
                    coordinator.getBusinessName());
            } // if (coordinator == null) else
            synchronized (session) {
                session.setAttribute(
                    Constants.REQUEST_CONTAINER,
                    session.getAttribute(Constants.REQUEST_CONTAINER));
            } // synchronized (session)
            /*
            try {
                EventExecutor.service(requestContainer, responseContainer);
            } // try
            catch (Exception ex) {
                TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.CRITICAL,
                    "AdapterPortlet::processAction:",
                    ex);
                emfErrorHandler.addError(
                    new EMFInternalError(EMFErrorSeverity.ERROR, ex));
            } // catch (Exception ex)
            */
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "AdapterPortlet::processAction: responseContainer",
                responseContainer);
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "AdapterPortlet::processAction: sessionContainer",
                requestContainer.getSessionContainer());
                
			if (serializeSession) {
				TracerSingleton.log(
					Constants.NOME_MODULO,
					TracerSingleton.DEBUG,
					"AdapterPortlet::processAction: sessionContainer size ["
						+ Serializer.serialize(
							requestContainer.getSessionContainer()).length
						+ "]");
			}
			
            String requestContainerName =
                Constants.REQUEST_CONTAINER
                    + requestContainer.hashCode();
            String responseContainerName =
                Constants.RESPONSE_CONTAINER
                    + responseContainer.hashCode();                           

            // set into the session new request and response container produced
            session.setAttribute(requestContainerName, requestContainer);
            session.setAttribute(responseContainerName, responseContainer);
            // set into the session the name of the new request and response container produced
            session.setAttribute(REQUEST_CONTAINER_NAME, requestContainerName);
            session.setAttribute(RESPONSE_CONTAINER_NAME, responseContainerName);
            // if the response is of type ActionResponse (the method has been called from the container)
            // set the name of the container like parameter of the request for the render method
            if(response instanceof ActionResponse) {
            	((ActionResponse)response).setRenderParameter(REQUEST_CONTAINER_NAME, requestContainerName);
            	((ActionResponse)response).setRenderParameter(RESPONSE_CONTAINER_NAME, responseContainerName);
            } 
            // if the response is of type RenderResponse the method has been called from the doRenderService
            // method and the container names must be returned to the method into an hashmap (it's not possible
            // to set them like parameters because the container will not call automatically the render service)
            else if(response instanceof RenderResponse){
            	map.put(REQUEST_CONTAINER_NAME, requestContainerName);
            	map.put(RESPONSE_CONTAINER_NAME, responseContainerName);
            }         
        } // try
        catch (Exception ex) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "AdapterPortlet::processAction: ",
                ex);
        } // catch (Excpetion ex) try
        finally {    
        	RequestContainer.delRequestContainer();
            if (monitor != null)
                monitor.stop();
        } // finally
        return map;
    } // public void processAction(ActionRequest request, ActionResponse)
    
    
   
   
    
    public void doView(RenderRequest request, RenderResponse response) {   	
    	PortletTracer.info(Constants.NOME_MODULO, "AdapterPortler", "doView", "Invocato");
    	doRenderService(request, response);
    } // public void doView(RenderRequest request, RenderResponse
    
    
    public void doEdit(RenderRequest request, RenderResponse response) {
    	PortletTracer.info(Constants.NOME_MODULO, "AdapterPortler", "doView", "Invocato");
    	doRenderService(request, response);
    } //public void doEdit(RenderRequest request, RenderResponse response) 
    
    
    
    public void doHelp(RenderRequest request, RenderResponse response) {
    	PortletTracer.info(Constants.NOME_MODULO, "AdapterPortler", "doView", " AdapterPortlet::doHelp Invocato");
    	doRenderService(request, response);
    } //public void doHelp(RenderRequest request, RenderResponse response)   
    
    
    
    
    
    
    
    
    
    
    
   
    
    public void doRenderService(RenderRequest request, RenderResponse response) {   	 	   	
        // set into threadLocal variables the jsr 168 portlet object 
    	PortletTracer.info(Constants.NOME_MODULO, "AdapterPortler", "doRenderService", " AdapterPortlet::doRenderService Invocato");
    	PortletSession portletSession = request.getPortletSession();
    	portletSession.setAttribute("PortalLocale", response.getLocale());
        PortletAccess.setPortletConfig(getPortletConfig());
        PortletAccess.setPortletRequest(request);
        PortletAccess.setPortletResponse(response);
        //PortletAccess.setPortalLocale(response.getLocale());
    	// get portlet mode
    	String portletMode = request.getPortletMode().toString();
    	TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "AdapterPortlet::doView: invocato per la portlet " + getPortletName() 
				+ " in modalita: " + portletMode);
    	PortletSession session = request.getPortletSession();
    	// get the name of the current request and reponse container which are contained into the session
    	// try to get the names from the attributes of the request (case of the loopback)
    	// if the names aren't defined into the attributes get them from the parameters of the request
    	// (normal case, not loopback)
    	String requestContainerName = "";
    	String responseContainerName = "";
    	requestContainerName =(String)request.getAttribute(REQUEST_CONTAINER_NAME);
    	responseContainerName = (String)request.getAttribute(RESPONSE_CONTAINER_NAME);
    	if( (requestContainerName==null) && (responseContainerName==null) ) {
	    	requestContainerName =(String)session.getAttribute(REQUEST_CONTAINER_NAME);
	    	responseContainerName = (String)session.getAttribute(RESPONSE_CONTAINER_NAME);
    	}
		// download from the session the object response container and request container
        RequestContainer requestContainer = null;
        ResponseContainer responseContainer = null;
		if (requestContainerName != null)
			requestContainer = (RequestContainer)session.getAttribute(requestContainerName);
		else 
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION, "AdapterPortlet::doView: requestContainerName nullo");
		if (responseContainerName != null) 
			responseContainer = (ResponseContainer)session.getAttribute(responseContainerName);
		else
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION, "AdapterPortlet::doView: responseContainerName nullo");
		// if request container is not null set into RequestContainer ThreadLocal variable
		if(requestContainer != null) 
			RequestContainer.setRequestContainer(requestContainer);
        Boolean isHttpResponseFreezed = null;
        if (responseContainer != null)
        	isHttpResponseFreezed = (Boolean) responseContainer.getAttribute(
                Constants.HTTP_RESPONSE_FREEZED);
        if ((isHttpResponseFreezed == null)
            || (!isHttpResponseFreezed.booleanValue())) {
        	String resource = null;
        	// if requestContainer and responseContainer are null meansthat is the first invocation of
        	// the doView method, so it's necessaty to get from the portlet configuration the type
        	// of service to execute
        	if (requestContainer == null && responseContainer == null) {
        		// ***************** START SERVICE ***********************************************
        		String serviceType = getPortletConfig().getInitParameter(SERVICE_TYPE);
        		String serviceName = getPortletConfig().getInitParameter(SERVICE_NAME);
        		if(serviceType==null) {
        			TracerSingleton.log(
        					Constants.NOME_MODULO,
            	            TracerSingleton.CRITICAL,
            	            "AdapterPortlet::doView: type of start service not find (ACTION/PAGE/JSP)");
        			//System.out.println("AdapterPortlet::doView: type of start service not find (ACTION/PAGE/JSP)");
        		}
        		if(!serviceType.equalsIgnoreCase(SERVICE_JSP) &&
        		   !serviceType.equalsIgnoreCase(SERVICE_ACTION) &&
        		   !serviceType.equalsIgnoreCase(SERVICE_PAGE) ) {
        				TracerSingleton.log(
        					Constants.NOME_MODULO,
            	            TracerSingleton.CRITICAL,
            	            "AdapterPortlet::doView: type of start service unknow (ACTION/PAGE/JSP)");
        				//System.out.println("AdapterPortlet::doView: type of start service unknow (ACTION/PAGE/JSP)");
        				return;
        		}
        		if(serviceName==null) {
        			TracerSingleton.log(
        					Constants.NOME_MODULO,
            	            TracerSingleton.CRITICAL,
            	            "AdapterPortlet::doView: name of start service not find");
        			System.out.println("AdapterPortlet::doView: name of start service not find");
        			return;
        		}		        		
        		if(serviceType.equalsIgnoreCase(SERVICE_JSP)) {
        			RouterPortlet router = new RouterPortlet(serviceName);
                    try {
                        router.route(getPortletContext(), request, response);   
            	    } catch (Exception ex) {
            	        TracerSingleton.log(
            	            Constants.NOME_MODULO,
            	            TracerSingleton.CRITICAL,
            	            "AdapterPortlet::doView: Error during the route to " + serviceName, ex);
            	        System.out.println("AdapterPortlet::doView: Error during the route to " 
            	        		            + serviceName + " (see log file)");
            	    }   
        		} else {
        			if(serviceType.equalsIgnoreCase(SERVICE_ACTION)) {
        				request.setAttribute("ACTION_NAME", serviceName);
        			} else {
        				request.setAttribute("PAGE", serviceName);
        			}
        			try{
        				HashMap map = (HashMap)processService(request, response);
                    	requestContainerName = (String) map.get(REQUEST_CONTAINER_NAME);
                    	responseContainerName = (String) map.get(RESPONSE_CONTAINER_NAME);
                    	// the new request and response name are set into the session and into the request
                    	// for the recursive call to the doRenderService
                    	session.setAttribute(REQUEST_CONTAINER_NAME, requestContainerName);
                        session.setAttribute(RESPONSE_CONTAINER_NAME, responseContainerName);
                        request.setAttribute(REQUEST_CONTAINER_NAME, requestContainerName);
                        request.setAttribute(RESPONSE_CONTAINER_NAME, responseContainerName);
                    	// recall the method for presentation
                        doRenderService(request, response); 
                        return;	                    	 
        			} catch (Exception e) {
        				TracerSingleton.log(
                	            Constants.NOME_MODULO,
                	            TracerSingleton.CRITICAL,
                	            "AdapterPortlet::doView: Error during the process of the "
								+ serviceType + " service " + serviceName, e);
        				System.out.println("AdapterPortlet::doView: Error during the process of the "
								+ serviceType + " service " + serviceName + " (see log file)");
        			}
        		} 
        		// *******************************************************************
        	} else {
	            String publisherType = null;
	            ArrayList resources = null;
	            Exception serviceException = (Exception)responseContainer.getAttribute(PORTLET_EXCEPTION);
	            Publisher publisher =
	                Publisher.getPublisher(
	                    requestContainer,
	                    responseContainer,
	                    serviceException);
	            if (publisher == null) {
	                publisherType = Constants.NOTHING_PUBLISHER_TYPE;
	                resources = null;
	            } // if (publisher == null)
	            else {
	                publisherType = publisher.getType();
	                if (publisherType
	                    .equalsIgnoreCase(Constants.NOTHING_PUBLISHER_TYPE))
	                    resources = null;
	                // if publisher is a loop publisher is necessary to recall the method process service 
	                // and do view (recursion) in order to exec the new request
	                else if (
	                    publisherType.equalsIgnoreCase(
	                        Constants.LOOP_PUBLISHER_TYPE)) {
	                    resources = new ArrayList();
	                    try { 
	                    	// se into the request a parameter loopback, the process service method
	                    	// will use it in order to know if is a loopback request or not
	                    	request.setAttribute(LOOPBACK, "loop");
	                    	// recall the process service, The name of the new request and response container
	                    	// are returned into an hashmap. Normally the names are set like parameters of the
	                    	// renderRequest from the process service method and when the container invokes
	                    	// the doView, doEdit, doHelp method these can get them. In this case calling the 
	                    	// method directly, the container cannot recall the doView, doEdit, doHelp so the 
	                    	// way is to set into the renderRequest object the names returned and call the 
	                    	// doRenderService (recursion)
	                    	HashMap map = (HashMap)processService(request, response);
	                    	requestContainerName = (String) map.get(REQUEST_CONTAINER_NAME);
	                    	responseContainerName = (String) map.get(RESPONSE_CONTAINER_NAME);
	                    	// the new request and response name are set into the session, override the 
	                    	// previous names 
	                    	session.setAttribute(REQUEST_CONTAINER_NAME, requestContainerName);
	                        session.setAttribute(RESPONSE_CONTAINER_NAME, responseContainerName);
	                        request.setAttribute(REQUEST_CONTAINER_NAME, requestContainerName);
	                        request.setAttribute(RESPONSE_CONTAINER_NAME, responseContainerName);
	                    	// recall the method for presentation
	                        doRenderService(request, response); 
	                        return;	                    	 
	                    } catch (Exception e) {
	                    	TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR, 
            						"AdapterPortlet::doRenderService: error during the execution of the loopback request", e);
	                    }
	                }
	                else {
	                    resources = publisher.getResources();
	                    if (resources.size() == 0) {
	                        publisherType = Constants.NOTHING_PUBLISHER_TYPE;
	                        resources = null;
	                    } // if (resources.size() == 0)
	                } // if (publisherType.equalsIgnoreCase(Constants.LOOP_PUBLISHER_TYPE)) else
	            } // if (publisher == null) else
	            if (publisherType
	                .equalsIgnoreCase(Constants.LOOP_PUBLISHER_TYPE)
	                || publisherType.equalsIgnoreCase(SERVLET_PUBLISHER_TYPE)
	                || publisherType.equalsIgnoreCase(JSP_PUBLISHER_TYPE)) {          	
	                Iterator iterator = resources.iterator();
	                SourceBean resourceSourceBean = null;                
	            	while (iterator.hasNext()) {
	            		resourceSourceBean = (SourceBean)iterator.next();
	            		if ( ((String)resourceSourceBean.getAttribute("mode")).equalsIgnoreCase(portletMode) )
	            			break;
	            	}
	            	if (resourceSourceBean == null) {
	            			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR, 
	            						"AdapterPortlet::doView: nessuna risorsa associata alla modalit� " + portletMode);
	            	} else 
	            		resource = (String)resourceSourceBean.getAttribute("resource");
	                        	                                
	                RouterPortlet router = new RouterPortlet(resource);
	                try {
	                    router.setParameter(Constants.REQUEST_CONTAINER, requestContainerName);
	                    router.setParameter(Constants.RESPONSE_CONTAINER,responseContainerName);
	                    router.route(getPortletContext(), request, response);   
	        	    } // try
	        	    catch (Exception ex) {
	        	        TracerSingleton.log(
	        	            Constants.NOME_MODULO,
	        	            TracerSingleton.CRITICAL,
	        	            "AdapterPortlet::doView: ",
	        	            ex);
	        	    } // catch (Excpetion ex) try      
	            } // if (publisherType.equalsIgnoreCase(Constants.LOOP_PUBLISHER_TYPE)
	              // || publisherType.equalsIgnoreCase(SERVLET_PUBLISHER_TYPE) ||
	              // publisherType.equalsIgnoreCase(JSP_PUBLISHER_TYPE))
	            else { 
	                response.setContentType(HTTP_CONTENT_TYPE);
	                Monitor renderingMonitor = null;
	                if (publisher == null)
	                    renderingMonitor =
	                        MonitorFactory.start(
	                            "view.portlet."
	                                + publisherType.toLowerCase());
	                else
	                    renderingMonitor =
	                        MonitorFactory.start(
	                            "view.portlet."
	                                + publisherType.toLowerCase()
	                                + "."
	                                + publisher.getName().toLowerCase());
	                try {
	                    response.getWriter().print(
	                        PresentationRendering.render(
	                            responseContainer,
	                            resources));
	                    response.getWriter().flush();
	                } // try
	                catch (Exception ex) {
	                	TracerSingleton.log(
	                            Constants.NOME_MODULO,
	                            TracerSingleton.DEBUG,
	                            "AdapterPortlet::doView:eccezzione", ex);
	                } // catch (Exception ex)
	                finally {
	                    renderingMonitor.stop();
	                } // finally
	            } // if (publisherType.equalsIgnoreCase(AF_PUBLISHER_TYPE)
	            // || publisherType.equalsIgnoreCase(SERVLET_PUBLISHER_TYPE) ||
	            // publisherType.equalsIgnoreCase(JSP_PUBLISHER_TYPE)) else
        	} //if (requestContainer == null && responseContainer == null) else
        } // if ((isHttpResponseFreezed == null) ||
        // (!isHttpResponseFreezed.getBoolean()))
        else
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.DEBUG,
                "AdapterPortlet::service: http response congelata");   
    } // public void doRenderService(RenderRequest request, RenderResponse
   

    
} // public class AdapterPortlet extends GenericPortlet
